#version 330

uniform vec4 light_diffusecomponent;
uniform vec4 light_ambientcomponent;
uniform vec4 light_specularcomponent;

uniform vec4 material_ambient;
uniform vec4 material_diffuse;
uniform vec4 material_specular;

uniform float specularcomponentfactor;

uniform sampler2D colormap;
uniform sampler2D normalmap;
in vec3 cameraview_out;

uniform sampler2D shadowmap;

in vec3 normal_out;
in vec4 vertex_out;
in vec2 tex_coords;
in vec4 shadow_tex_coords;

out vec4 fragColor;

in vec3 lightdir;
in vec3 lightpos;
in vec3 viewdir;

void main (void)  
{  
	vec3 L = normalize(lightdir);
	vec3 N = normalize(texture2D(normalmap, tex_coords).xyz * 2.0 - 1.0);
	vec3 V = normalize(cameraview_out);
	vec3 R = normalize(-reflect(L, N));
	
	float nDotL = max(0.0, dot(N, L));
	float rDotV = max(0.0, dot(R, V));
	
	vec4 ambient = light_ambientcomponent * material_ambient;
	vec4 diffuse = light_diffusecomponent * material_diffuse * nDotL;
	vec4 specular = light_specularcomponent * material_specular * pow(rDotV, specularcomponentfactor);
	vec4 texel = texture2D(colormap, tex_coords);

	//shadow mapping
	vec3 sm_tex_coords = shadow_tex_coords.xyz/shadow_tex_coords.w;
	sm_tex_coords.z -= 0.005;
	float depth = texture(shadowmap, sm_tex_coords.xy).r;
	float inShadow = (depth < sm_tex_coords.z) ? 0.2 : 1.0;
	//if (shadow_tex_coords.w < 0.0)
	//	inShadow = 1.0;
		
    fragColor = (((ambient + diffuse) * texel) + specular) * inShadow;
}