#version 330

uniform mat4 projection;
uniform mat4 view;
uniform mat4 model;
uniform mat4 bias;
uniform mat4 lightview;
uniform mat4 shadowprojection;

uniform vec3 cameraview;

uniform vec3 lightposition;

in vec4 vertex;
in vec3 normal;
in vec4 tangent;
in vec2 tex_coords_in;

out vec3 normal_out;
out vec4 vertex_out;
out vec2 tex_coords;
out vec4 shadow_tex_coords;
out vec3 cameraview_out;

out vec3 lightdir;

void main(void)
{
	normal_out = (model * vec4(normal, 0.0)).xyz;
	
	vertex_out = model * vertex;

	vec3 n = normalize(normal_out);
	vec3 t = normalize((model*vec4(tangent.xyz, 0.0)).xyz);
	vec3 b = cross(n, t);

	mat3 tbnMatrix = mat3(t.x, b.x, n.x,
                          t.y, b.y, n.y,
                          t.z, b.z, n.z);

	vec4 pos = view * model * vertex;

	lightdir = lightposition - vertex_out.xyz;
	lightdir = tbnMatrix * lightdir;

	cameraview_out = tbnMatrix * normalize(cameraview);
	cameraview_out = normalize(cameraview_out);

	tex_coords = tex_coords_in;

	shadow_tex_coords = bias * shadowprojection * lightview * model * vertex;
	
	gl_Position = projection * view * model * vertex;
}  